// -------------------------------------------------------------------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tTex <string uiname="Texture Transform";>;
 
// control params

float Opacity = 0.5;
float noiseScale = 0.5;

// textures and samplers
texture texBase;
sampler2D sampBase = sampler_state
{
    Texture = <texBase>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Linear;
    MipFilter = Linear;
};

texture texBlend ;
sampler2D sampBlend = sampler_state
{
    Texture = <texBlend>;
    AddressU = Clamp;
    AddressV = Clamp;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = Point;
};

struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float2 TexC : TEXCOORD0;
};

// ================================================================
// vertex shader
// ================================================================

VS_OUTPUT VS(
    float4 Pos  : POSITION,
    float2 TexC : TEXCOORD)
{
    //inititalize all fields of output struct with 0
    VS_OUTPUT Out = (VS_OUTPUT)0;

    //transform position
    Pos = mul(Pos, tWVP);
    
    //transform texturecoordinates
    TexC = mul(TexC, tTex);

    Out.Pos  = Pos;
    Out.TexC = TexC;

    return Out;
}



// ================================================================
// Pixel Shader
// ================================================================

static const float4 white = {1., 1., 1., 1.};
float3 lumCoefficient = {0.2125, 0.7154, 0.0721};

float3 mix(float3 x, float3 y, float a)
{
   return float3(x.r * (1.-a) + y.r * a, x.g * (1.-a) + y.g * a, x.b * (1.-a) + y.b * a);
}

float4 mix(float4 x, float4 y, float a)
{
   return float4(x.r * (1.-a) + y.r * a, x.g * (1.-a) + y.g * a, x.b * (1.-a) + y.b * a, x.a * (1.-a) + y.a * a);
}


float4 addClamp(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    float4 result = clamp(blend + base, 0.0, 1.0);
    return float4((mix(base, result, Opacity)).rgb, 1.);
}

float4 _normal(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return mix(base, blend, Opacity);
}

float4 average(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return float4(((base + blend ) * 0.5).rgb, 1.);
}

/*float4 disolve(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    float _noise = (noise(Tex * noiseScale) + 1.) * 0.5;

    return (_noise < Opacity) ? blend : base;
}*/

float4 behind(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return (base.a == 0.0) ? blend : base;
}

/*float4 clear(float2 Tex : TEXCOORD0 ) : COLOR0
{
    //float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return float4(blend.rgb, 0.);
}*/

float4 darken(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return min(blend, base);
}

float4 lighten(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return max(blend, base);
}

float4 multiply(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return blend * base;
}

float4 screen(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return float4((white - ((white-blend) * (white-base))).rgb, 1.);
}

float4 colorBurn(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return float4((white - (white - base) / blend).rgb, 1.);
}

float4 colorDodge(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return float4( (base / (white - blend)).rgb, 1.);
}

float4 overlay(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    
    float luminance = dot(base, lumCoefficient);
    float4 result = 0.;
    if(luminance < 0.45)
        result = 2. * blend * base;
    else if(luminance > 0.55)
         result = white - 2. * (white - blend) * (white - base);
    else
         result = mix(2 * blend * base, white - 2. * (white - blend) * (white - base), (luminance-0.45)*10.);
    return float4(result.rgb, 1.);
}

float4 softLight(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return float4((2.* base * blend + base * base - 2. * base * base * blend).rgb, 1.);
}

float4 hardLight(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);

    float luminance = dot(blend, lumCoefficient);
    float4 result = 0.;

    if(luminance < 0.45)
        result = 2. * blend * base;
    else if(luminance > 0.55)
         result = white - 2. * (white - blend) * (white - base);
    else
         result = mix(2 * blend * base, white - 2. * (white - blend) * (white - base), (luminance-0.45)*10.);

    return float4(result.rgb, 1.);
}

float4 add(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return blend + base;
}

float4 subtract(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return float4(base.rgb - blend.rgb, 1.);
}

float4 difference(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return float4(abs(blend - base).rgb, 1.);
}

float4 inverseDifference(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return float4( (white - abs(white - base - blend).rgb), 1.);
}

float4 exclusion(float2 Tex : TEXCOORD0 ) : COLOR0
{
    float4 base = tex2D(sampBase, Tex);
    float4 blend = tex2D(sampBlend, Tex);
    return float4( (base + blend - (2. * base * blend)).rgb, 1.);
}

// ================================================================
// Techniques
// ================================================================


technique AddClamp
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 addClamp();
        ZEnable = false;
    }
}

technique _Normal
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 _normal();
        ZEnable = false;
    }
}

technique Average
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 average();
        ZEnable = false;
    }
}

/*technique Disolve
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 disolve();
        ZEnable = false;
    }
}*/

technique Behind
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 behind();
        ZEnable = false;
    }
}

/*technique Clear
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 clear();
        ZEnable = false;
    }
}*/

technique Darken
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 darken();
        ZEnable = false;
    }
}

technique Lighten
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 lighten();
    }
}

technique Multiply
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 multiply();
    }
}

technique Screen
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 screen();
    }
}

technique ColorBurn
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 colorBurn();
    }
}

technique ColorDodge
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 colorDodge();
    }
}

technique Overlay
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 overlay();
    }
}

technique SoftLight
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 softLight();
        ZEnable = false;
    }
}

technique HardLight
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 hardLight();
        ZEnable = false;
    }
}

technique Add
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 add();
        ZEnable = false;
    }
}

technique Subtract
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 subtract();
        ZEnable = false;
    }
}

technique Difference
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 difference();
        ZEnable = false;
    }
}

technique InverseDifference
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 inverseDifference();
        ZEnable = false;
    }
}

technique Exclusion
{
    pass p0
    {
        VertexShader = compile vs_2_0 VS();
        PixelShader = compile ps_2_0 exclusion();
        ZEnable = false;
    }
}
