// Gouraud-Lighting : calculate color (lighted) for the vertices and interpolate over the pixels

// -------------------------------------------------------------------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tWV: WORLDVIEW;
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)

//material properties
float4 mAmb  : COLOR <String uiname="Ambient Color";>  = {  0.26058,    0.54742,    0.26058,    1.00000  };
float4 mDiff : COLOR <String uiname="Diffuse Color";>  = {  0.03943,    0.63600,    0.00000,    1.00000  };
float4 mSpec : COLOR <String uiname="Specular Color";> = {  0.75466,    1.00000,    0.72800,    1.00000  };
float mPower <String uiname="Power"; float uimin=0.0;> = 10.0;   //shininess of specular highlight

//light properties
float4 lAmb  : COLOR <String uiname="Ambient Light";>  = {  0.26440,    0.26760,    0.26493,    1.00000  };
float4 lDiff : COLOR <String uiname="Diffuse Light";>  = {  0.56848,    0.56973,    0.56627,    1.00000  };
float4 lSpec : COLOR <String uiname="Specular Light";> = {  1.00000,    1.00000,    0.72800,    1.00000  };
float3 lDir <string uiname="Light Direction";> = {   0.577,   -0.577,   0.577  };          //Light Direction ( in view space !! )

float3 pos01;
float3 pos02;
float3 pos03;

float yoffset;

// -------------------------------------------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// -------------------------------------------------------------------------------------------------------------------------------------

//data that ist returned by the vertexshader
struct VS_OUTPUT
{
    float4 Pos: POSITION;
    float4 Diff: COLOR0;
    float4 Spec: COLOR1;
    float4 Amb : TEXCOORD0;
};

VS_OUTPUT VS(
    float4 Pos: POSITION,
    float3 Norm: NORMAL)
{
    float _min = 0.18;
    float _max = 0.8;

    //inititalize all fields of output struct with 0
    VS_OUTPUT Out = (VS_OUTPUT)0;

    Pos = mul(Pos, tW);
    
    float3 oPos = Pos.xyz;

    pos01 = clamp(pos01, _min, _max);
    float3 d01 = distance( oPos, pos01 );
    float3 dir01 = pos01-oPos;
    Pos.x = Pos.x-dir01.x*1/d01.x;
    Pos.y = Pos.y-log(clamp(d01.y, 0, 1.5));   //Pos.y = Pos.y-clamp(0.3/d.y, 0.3, 1);
    Pos.z = Pos.z-dir01.z*1/d01.z;

    pos02 = clamp(pos02, _min, _max);
    float3 d02 = distance( oPos, pos02 );
    float3 dir02 = pos02-oPos;
    Pos.x = Pos.x-dir02.x*1/d02.x;
    Pos.y = Pos.y-log(clamp(d02.y, 0, 1.5));   //Pos.y = Pos.y-clamp(0.3/d.y, 0.3, 1);
    Pos.z = Pos.z-dir02.z*1/d02.z;
    
    pos03 = clamp(pos03, _min, _max);
    float3 d03 = distance( oPos, pos03 );
    float3 dir03 = pos03-oPos;
    Pos.x = Pos.x-dir03.x*1/d03.x;
    Pos.y = Pos.y-log(clamp(d03.y, 0, 1.5));   //Pos.y = Pos.y-clamp(0.3/d.y, 0.3, 1);
    Pos.z = Pos.z-dir03.z*1/d03.z;
    
    /*Pos.y += yoffset;    */
    
    //Pos.xyz = Pos.xyz+log(clamp(d.xyz, 0, 1.5));

    float3 L = -lDir;
    Norm = normalize(mul(Norm, tWV));
    //normal (view space)
    float3 N = Norm.xyz;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(N, L) * N - L);
    //view direction (view space)
    float3 V = -normalize(Pos.xyz);
    
    L = normalize(L);
    float4 diff = max(dot(N, L), 0) * mDiff * lDiff;
    diff.a = 1.0;
    float4 spec = pow(max(dot(R, V),0), mPower) * mSpec * lSpec;
    
    //position (projected)
    Pos = mul(Pos, tV);
    Pos = mul(Pos, tP);
    
    Out.Pos  = Pos;
    Out.Diff = diff;
    Out.Spec = spec;
    Out.Amb  = lAmb * mAmb;
    
    return Out;    
}

// -------------------------------------------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// -------------------------------------------------------------------------------------------------------------------------------------

float4 PSGouraud(
    float4 Diff : COLOR0,
    float4 Spec : COLOR1,
    float4 Amb  : TEXCOORD0): COLOR
{
    return float4((Diff + Spec + Amb).rgb, 1.);
}

float4 PSDiffuse(float4 Diff: COLOR0 ): COLOR
{
    return Diff;
}

float4 PSSpecular(float4 Spec: COLOR1): COLOR
{
    return Spec;
}

float4 PSAmbient(float4 Amb: TEXCOORD0): COLOR
{
    return Amb;
}

// -------------------------------------------------------------------------------------------------------------------------------------
// TECHNIQUES:
// -------------------------------------------------------------------------------------------------------------------------------------

technique TGouraud
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader = compile ps_2_0 PSGouraud();
    }
}


technique TDiffuseOnly
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader = compile ps_1_4 PSDiffuse();
    }
}

technique TSpecularOnly
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader = compile ps_1_4 PSSpecular();
    }
}

technique TAmbientOnly
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader = compile ps_2_0 PSAmbient();
    }
}

